#!/bin/sh

####################################################
#   /sdcard/update.tgz
#   /sdcard/update.md5  [optional]
####################################################
SDCARD_PATH=/sdcard
PACKAGE_NAME=update.tgz
MD5_CHKSUM_UPDATE=$SDCARD_PATH/update.md5
PATH_UPDATE=$SDCARD_PATH/update

####################################################
#   update/IMAGE/xxxV1.0.x.bin
#   update/IMAGE/image.md5
####################################################
IMAGE_PATH=$PATH_UPDATE/IMAGE       ##upgrade image for the whole nand flash.
MD5_CHKSUM_IMAGE=$IMAGE_PATH/image.md5

####################################################
#   update/APP/app.tgz :
#   update/APP/app.md5
####################################################
APP_NAME=app.tgz
APP_PATH=$PATH_UPDATE/APP           ##update files for /opt/app/
MD5_CHKSUM_APP=$APP_PATH/app.md5
APP_PATH_DEV=/opt/
APP_OPT_PATH=/opt/app/bin/


## return 0 on success, and 1 on error.

func_check_md5_update()
{
    cd $SDCARD_PATH
    echo -e "$MD5_CHKSUM_UPDATE"
    if [ -f $MD5_CHKSUM_UPDATE ]; ##check update.md5 existence.
    then
        echo "`cat $MD5_CHKSUM_UPDATE`"
        UPDATE_CHECKSUM=`awk '{print $1}' $MD5_CHKSUM_UPDATE`
        UPDATE_PACKAGENAME=`awk '{print $2}' $MD5_CHKSUM_UPDATE`
        echo "+++ update check_sum     : $UPDATE_CHECKSUM"
        echo "+++ update package name  : $UPDATE_PACKAGENAME"

        if [ -f $SDCARD_PATH/$UPDATE_PACKAGENAME ] && [ $PACKAGE_NAME == $UPDATE_PACKAGENAME ]; ##check update.tgz existence.
        then
            /usr/bin/md5sum $PACKAGE_NAME > tmp.md5  ##generate new md5 checksum.
            echo -e "`cat tmp.md5`"
            NEW_CHECK_SUM=`awk '{print $1}' tmp.md5`
            echo -e "+++ new check_sum  : $NEW_CHECK_SUM"

            if [ $UPDATE_CHECKSUM == $NEW_CHECK_SUM ];
            then
                echo -e "++++++++ [update: $UPDATE_PACKAGENAME] md5 matched success ++++++++"
                return 0;    ##succuss.
            else
                echo -e "!!!!!!!! [update: $UPDATE_PACKAGENAME] md5 check failed !!!!!!!!"
                return 1;   ##check md5 failed.
            fi
        else ##update.tgz not existed.
            echo -e "!!! Error, $SDCARD_PATH/$UPDATE_PACKAGENAME is not found !!!!!!"
            return 1;
        fi
    else ##update.md5 not existed.
        #echo -e "!!! Error, $MD5_CHKSUM_UPDATE is not found !!!!!!"
        #return 1;
        echo -e "!!!!!! Warning, $MD5_CHKSUM_UPDATE is not found !!!!!!"
        return 0; ##一级校验, update.md5不是必须的.
    fi
}


func_do_uncompress()
{
    cd $SDCARD_PATH
    compressed_package=$SDCARD_PATH/$PACKAGE_NAME
    echo "$compressed_package"
    if [ -f $compressed_package ]; ##check update.tgz existence.
    then
        echo -e "+++ uncompress $compressed_package ......"
        uncompress_result=`/bin/tar -xvf $compressed_package -C $SDCARD_PATH`
        return 0    ##succuss.
    else
        echo -e "!!! Error, $SDCARD_PATH/$PACKAGE_NAME is not found !!!!!!"
        return 1;
    fi
}


func_upgrade_image()
{
    echo -e "+++ IMAGE path : $IMAGE_PATH"
    echo -e "+++ IMAGE md5  : $MD5_CHKSUM_IMAGE"
    if [ -d $IMAGE_PATH ] && [ -f $MD5_CHKSUM_IMAGE ];
    then
        echo "`cat $MD5_CHKSUM_IMAGE`"
        IMAGE_CHECKSUM=`awk '{print $1}' $MD5_CHKSUM_IMAGE`
        IMAGE_PACKAGENAME=`awk '{print $2}' $MD5_CHKSUM_IMAGE`
        echo "+++ image check_sum     : $IMAGE_CHECKSUM"
        echo "+++ image package name  : $IMAGE_PACKAGENAME"

        if [ -f $IMAGE_PATH/$IMAGE_PACKAGENAME ]; ##check xxxV1.0.x.bin existence.
        then
            cd $IMAGE_PATH
            /usr/bin/md5sum $IMAGE_PACKAGENAME > image_tmp.md5  ##generate new md5 checksum.
            echo -e "`cat image_tmp.md5`"
            NEW_IMG_CHECKSUM=`awk '{print $1}' image_tmp.md5`
            echo -e "+++ new image check_sum  : $NEW_IMG_CHECKSUM"

            if [ $IMAGE_CHECKSUM == $NEW_IMG_CHECKSUM ];
            then
                echo -e "++++++++ [image: $IMAGE_PACKAGENAME] md5 matched success ++++++++"
                if [ -d $SDCARD_PATH ];
                then
                    echo -e "   >>>>>> copy $IMAGE_PACKAGENAME to $SDCARD_PATH >>>>>>    "
                    /bin/cp $IMAGE_PATH/$IMAGE_PACKAGENAME $SDCARD_PATH
                    /bin/sync
                    return 0;    ##succuss.
                else ##sdcard not found.
                    echo -e "!!!!!!!! Warning, $SDCARD_PATH is not existed !!!!!!!!"
                    return 1;
                fi
            else
                echo -e "!!!!!! [image: $IMAGE_PACKAGENAME] md5 check failed !!!!!!"
                return 1;   ##check md5 failed.
            fi
        else ##image file not existed.
            echo -e "!!! Error, $IMAGE_PATH/$UPDATE_PACKAGENAME is not found !!!!!!"
            return 1;
        fi
    else ##image directory is empty, no need to upgrade.
        echo -e "!!! image not existed, there no need to upgrade the whole nand flash !!!"
        return 1;
    fi
}


func_upgrade_app()
{
    echo -e "+++ APP path : $APP_PATH"
    echo -e "+++ APP md5  : $MD5_CHKSUM_APP"
    if [ -d $APP_PATH ] && [ -f $MD5_CHKSUM_APP ];
    then
        echo "`cat $MD5_CHKSUM_APP`"
        APP_CHECKSUM=`awk '{print $1}' $MD5_CHKSUM_APP`
        APP_PACKAGENAME=`awk '{print $2}' $MD5_CHKSUM_APP`
        echo "+++ image check_sum     : $APP_CHECKSUM"
        echo "+++ image package name  : $APP_PACKAGENAME"
        if [ -f $APP_PATH/$APP_PACKAGENAME ]; ##check app.tgz existence.
        then
            cd $APP_PATH
            /usr/bin/md5sum $APP_PACKAGENAME > app_tmp.md5  ##generate new md5 checksum.
            echo -e "`cat app_tmp.md5`"
            NEW_APP_CHECKSUM=`awk '{print $1}' app_tmp.md5`
            echo -e "+++ new app check_sum  : $NEW_APP_CHECKSUM"

            if [ $APP_CHECKSUM == $NEW_APP_CHECKSUM ];
            then
                echo -e "++++++++ [APP: $APP_PACKAGENAME] md5 matched success ++++++++"
                if [ -d $APP_PATH_DEV ];
                then
		    cd $APP_OPT_PATH
                    echo -e "   >>>>>> cover files into $APP_PATH_DEV >>>>>>    "
		    /bin/rm -rf `ls | grep -v 'cv25_check_upgrade.sh' | grep -v 'S13-driver' `
                    /bin/tar -xvf $APP_PATH/$APP_PACKAGENAME -C $APP_PATH_DEV
                    /bin/sync
                    /bin/chmod -R a+x $APP_PATH_DEV
                    return 0;    ##succuss.
                fi
            else
                echo -e "!!!!!! Error, [APP: $APP_PACKAGENAME] md5 check failed !!!!!!"
                return 1;   ##check md5 failed.
            fi
        else ##app.tgz not existed.
            echo -e "!!! Error, $APP_PATH/$APP_PACKAGENAME is not found !!!!!!"
            return 1;
        fi
    else ##APP directory is empty, no need to update app.
        echo -e "!!! APP is empty, there no need to update file in /opt/app !!!"
        return 0;
    fi
}


do_system_restart()
{
    /bin/sleep 1
    /bin/sync
    /bin/rm -f $MD5_CHKSUM_UPDATE
    /bin/rm -f $SDCARD_PATH/$PACKAGE_NAME
    echo -e
    echo -e ">>>>>> System Restart Now...... "
    echo -e
    /sbin/reboot -f
}


## main ##
/bin/rm -f $SDCARD_PATH/SDupgrade*
/bin/rm -rf $SDCARD_PATH/update

/bin/sleep 1
/bin/sync

echo -e ">>> $SDCARD_PATH/$PACKAGE_NAME"
if [ -f $SDCARD_PATH/$PACKAGE_NAME ]; ##check update.tgz existence.
then
    echo -e "++++++++++++++ CV25 Check Upgrade Start +++++++++++++++";
else
    echo -e "++++++++++ [update.tgz] Not found, there is no need to upgrade ++++++++++"
    exit
fi

echo "==> step 1: check md5 for update package......"
cd $SDCARD_PATH
func_check_md5_update
if [ $? == 0 ];
then
    echo -e ">>>>>> check update md5 success <<<<<<"
else
    echo -e "!!! Error, ret_val = $?, check md5 for update package failed, exit !!!!!!"
    exit
fi

echo "==> step 2: uncompress......"
func_do_uncompress
if [ $? == 0 ];
then
    echo -e ">>>>>> do uncompress success <<<<<<"
else
    echo -e "!!! Error, ret_val = $?, uncompress update package failed, exit !!!!!!"
    exit
fi

echo "==> step 3: check the image and upgrade the whole nand flash partition......"
func_upgrade_image
if [ $? == 0 ];
then
    echo -e ">>>>>> check image success <<<<<<"
    do_system_restart
    exit
else
    #echo -e "!!! Error, ret_val = $?, check partition image failed, exit !!!!!!"
    echo -e "!!!!!! partition image not found, there is no need to upgrade the flash parition !!!!!!"
fi

echo "==> step 4: check the app files andd update thems in /opt/app/......"
func_upgrade_app
if [ $? == 0 ];
then
    echo -e ">>>>>> update app files success <<<<<<"
    do_system_restart
    exit
else
    echo -e "!!!!!! app not found, there is no need to upgrade !!!!!!"
fi


