#! /bin/sh

#gpio_101 0
#gpio_102 night:1	day:0
#gpio14(in) high:1(写)	
#gpio98(in) reboot:0(读)
#gpio_10红	gpio_19绿	gpio_20蓝

#宏定义
day=0			#白天
night=1			#夜晚
ADC1_addr=0xE4003154	#ADC1地址
sampleValueMax=100	#采样值上限(切白天)
sampleValueMin=10	#采样值下限(切晚上)
reset_timeout=15	#gpio_98次数触发值
ircut_timeout=15	#IR-cut次数触发值
IMAGE_CMD_FIFO="/tmp/cmd_fifo"
AppConfig="/opt/app/config/appConfig.cfg"
FORCE_RGB="/opt/app/config/FORCE_RGB.flag"
YSGN="/opt/app/config/YSGN.flag"
Offset=82		#appConfig.cfg中mSaturation参数偏移量
LED_file="/tmp/LED_states"


#变量区
cur_Mode=0		#0:day -- 1:night
sampleValue=0		#IR-cut -- ADC1采样值
ircut_flag=0		#IR-cut -- Flag
ircut_time=0		#IR-cut -- 次数检测
gpio_reset=0		#gpio_98 -- 状态
reset_flag=0		#gpio_98 -- 长按Flag
reset_time=0		#gpio_98 -- 次数检测
led_states=0		#led -- 当前状态
last_led_states=0	#led -- 上次状态
green_flashing=0	#绿灯闪烁 -- Flag
green_state=0		#绿灯状态


#获取Saturation值
get_saturation(){
	num=0x`hexdump -n 1 -C -s $Offset $AppConfig | awk '{print $2}'`
	num=$(($num))
	echo $num
}

#设置Saturation值
set_saturation(){
	echo Saturation,$1 > $IMAGE_CMD_FIFO
}

#IR-cut 检测
check_ircut(){
	
	ircut_time=$(($ircut_time + 1))
	
	if [ $ircut_time -ge $ircut_timeout ];then
		ircut_flag=1		
	fi	

	if [ $ircut_flag -eq 1 ];then
		ircut_flag=0

		if [ $cur_Mode -eq $day ];then
			set_saturation 0
			cur_Mode=$night
			/usr/local/bin/amba_debug -g 102 -d 1
			/usr/local/bin/amba_debug -g 14 -d 1
			echo "cur_Mode :night --- sampleValue :$sampleValue"
		else
			set_saturation `get_saturation`
			cur_Mode=$day
			/usr/local/bin/amba_debug -g 102 -d 0
			/usr/local/bin/amba_debug -g 14 -d 0
			echo "cur_Mode :day --- sampleValue :$sampleValue"
		fi

	fi

}


#合法性检测
check_adc_legal(){
	#Max
	sampleValueMax=`cat -n ircut_config | grep "sampleValueMax" | awk '{print $3}'`
	sampleValueMax=$(($sampleValueMax + 0))
	#echo "sampleValueMax : $sampleValueMax"	

	#Min
	sampleValueMin=`cat -n ircut_config | grep "sampleValueMin" | awk '{print $3}'`
	sampleValueMin=$(($sampleValueMin + 0))
	#echo "sampleValueMin : $sampleValueMin"

	#Min>Max?1:0
	if [ $sampleValueMin -gt $sampleValueMax ];then
		sampleValue=$sampleValueMin
		sampleValueMin=$sampleValueMax
		sampleValueMax=$sampleValue
	fi	

	#ad
	#sampleValue=`cat -n ircut_config | grep "ad_value" | awk '{print $3}'`
	sampleValue=`/usr/local/bin/amba_debug -r $ADC1_addr | awk '{printf $2}'`
	sampleValue=$(($sampleValue + 0))
	#echo "sampleValue : $sampleValue"

	#check
	if [ $cur_Mode -eq $night ]&&[ $sampleValue -gt $sampleValueMax ];then
		check_ircut
	elif [ $cur_Mode -eq $night ]&&[ -e $FORCE_RGB ];then
		check_ircut
    elif [ $cur_Mode -eq $night ]&&[ -e $YSGN ];then
        check_ircut
	elif [ $cur_Mode -eq $day ]&&[ $sampleValue -lt $sampleValueMin ]&&[ ! -e $FORCE_RGB ]&&[ ! -e $YSGN ];then
		check_ircut
	else
		ircut_time=0
	fi	
}


#gpio_98 reboot
gpio_reboot(){
	gpio_reset=`/usr/local/bin/amba_debug -g 98 | grep data | awk '{print $2}'`
	gpio_reset=$(($gpio_reset + 0))
	#echo "gpio_reset : $gpio_reset"

	if [ $gpio_reset -eq 0 ];then
		reset_time=$(($reset_time + 1))

		if [ $reset_time -ge $reset_timeout ];then
			reset_flag=1
		fi

		if [ $reset_flag -eq 1 ];then
			reset_flag=0
			reset_time=0
			rm /opt/app/config/sta_info
			#rm /opt/app/config/appConfig.cfg
			rm -rf /opt/app/config/motor.ini
			rm -rf /opt/app/config/*.flag
			#rm -rf /opt/app/config/recorder_face.db
			sync
			/usr/bin/aplay /opt/app/voice/ch/boot.pcm
			echo "gpio_98 reboot"	
			sleep 1
			reboot		
		fi
	else
		reset_time=0
	fi
}

LED_open(){
	#echo " --- $1 --- $2 --- $3"
	/usr/local/bin/amba_debug -g 10 -d $1	#红灯
	/usr/local/bin/amba_debug -g 19 -d $2	#绿灯
	/usr/local/bin/amba_debug -g 20 -d $3	#蓝灯
}

#LED
Check_led(){
	if [ -f "$LED_file" ];then
		led_states=`cat -n $LED_file | grep "LED_states" | awk '{print $3}'`
		led_states=$(($led_states + 0))

		if [ $last_led_states -ne $led_states ];then
			last_led_states=$led_states
			green_flashing=0
			#echo "last_led_states : $last_led_states -- led_states : $led_states"
			case $led_states in
			    1)
				echo "1 : red on"
				LED_open 1 0 0
				;;
			    2)
				echo "2 : green flashing"
				LED_open 0 0 0
				green_flashing=1	#绿灯闪烁
				;;
			    3)
				echo "3 : green on"
				LED_open 0 1 0
				;;
			    4)
				echo "4 : blue on"
				LED_open 0 0 1
				;;
	  		    5)
				echo "5 : all off"
				LED_open 0 0 0
				;;
			    *)
				echo "err : [other num] red on"
				LED_open 1 0 0
			esac
		fi

		if [ $green_flashing -ge 1 ];then
			green_flashing=$(($green_flashing+1))
			if [ $green_flashing -ge 3 ];then
				green_flashing=1
				green_state=$((!$green_state))
				/usr/local/bin/amba_debug -g 19 -d $green_state	
				#echo "green_state : $green_state"		
			fi
		fi
	else
		#echo "err : [no file] red on"
		last_led_states=0
		LED_open 1 0 0
	fi
}

while [ "1" = "1" ]
do	
	Check_led
	#sleep 1
	gpio_reboot
	check_adc_legal
	usleep 100000	#0.1s

done



echo " ----------- OVER"






